@@ -3,7 +3,7 @@ import json
 import logging
 import time
 import timeit
-from typing import Any
+from typing import Any, List
 
 import boto3
 import s3path
@@ -15,29 +15,57 @@ log = logging.getLogger(__name__)
 # Ref: https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/glue.html
 
 
-def ensure_and_run_s3_tables_crawler(crawler: str, s3_path: s3path.S3Path, **kwargs: Any) -> None:
-    """Ensure and run the specified AWS Glue crawler to crawl the top-level tables in the given S3 path.
+def delete_database(**kwargs: Any) -> None:
+    """Safely delete the specified AWS Glue database.
+
+    At minimum the `Name` keyword argument is required.
+
+    :param kwargs: These are forwarded to `delete_database`.
+    """
+    name = kwargs["Name"]
+    client = boto3.client("glue")
+
+    try:
+        response = client.delete_database(**kwargs)
+    except client.exceptions.EntityNotFoundException:
+        log.info(f"AWS Glue does not have a database named {name}.")
+    else:
+        assert response["ResponseMetadata"]["HTTPStatusCode"] == 200
+        log.info(f"Deleted AWS Glue database {name}.")
+
+
+def ensure_and_run_crawler(**kwargs: Any) -> List[dict]:
+    """Return the list of database tables after ensuring and running the specified AWS Glue crawler with the given configuration.
 
     :param kwargs: These are indirectly forwarded to `create_crawler`.
     """
+    crawler_name = kwargs["Name"]
+    ensure_crawler(**kwargs)
+    run_crawler(crawler_name)
+
+    db_name = kwargs.get("DatabaseName") or crawler_name
+    tables = get_tables(DatabaseName=db_name)
+    table_names = [t["Name"] for t in tables]
+    log.info(f"After crawling {crawler_name}, {len(tables)} exist: {', '.join(table_names)}")
+    return tables
+
+
+def ensure_and_run_s3_tables_crawler(crawler: str, s3_path: s3path.S3Path, **kwargs: Any) -> List[dict]:
+    """Return the list of database tables after ensuring and running the specified AWS Glue crawler to crawl the top-level tables in the given S3 path.
+
+    :param kwargs: These are indirectly forwarded to `create_crawler`.
+    """
+    # Ref: https://stackoverflow.com/a/67185483/
     assert all(k not in kwargs for k in ("Name", "Targets"))
 
     s3_crawl_targets = [{"Path": path_to_uri(p)} for p in s3_path.glob("*")]  # For correct crawling, each table's path is independently included.
     assert s3_crawl_targets
     log.info(f"{len(s3_crawl_targets)} S3 paths are to be crawled, the first of which is {s3_crawl_targets[0]['Path']}.")
 
-    ensure_and_run_crawler(
+    tables = ensure_and_run_crawler(
         Name=crawler, Targets={"S3Targets": s3_crawl_targets}, Configuration=json.dumps({"Version": 1.0, "Grouping": {"TableGroupingPolicy": "CombineCompatibleSchemas"}}), **kwargs
     )
-
-
-def ensure_and_run_crawler(**kwargs: Any) -> None:
-    """Ensure and run the specified AWS Glue crawler with the given configuration.
-
-    :param kwargs: These are indirectly forwarded to `create_crawler`.
-    """
-    ensure_crawler(**kwargs)
-    run_crawler(kwargs["Name"])
+    return tables
 
 
 def ensure_crawler(**kwargs: Any) -> None:
@@ -70,16 +98,15 @@ def ensure_crawler(**kwargs: Any) -> None:
     assert response["ResponseMetadata"]["HTTPStatusCode"] == 200
 
 
-def remove_database(name: str) -> None:
-    """Safely delete the specified AWS Glue database."""
-    client = boto3.client("glue")
-    try:
-        response = client.delete_database(Name=name)
-    except client.exceptions.EntityNotFoundException:
-        log.info(f"AWS Glue does not have a database named {name}.")
-    else:
-        assert response["ResponseMetadata"]["HTTPStatusCode"] == 200
-        log.info(f"Deleted AWS Glue database {name}.")
+def get_tables(**kwargs: Any) -> List[dict]:
+    """Return the list of tables in the specified AWS Glue database.
+
+    At minimum the `DatabaseName` keyword argument is required.
+
+    :param kwargs: These are forwarded to `get_tables`.
+    """
+    assert "DatabaseName" in kwargs
+    return boto3.client("glue").get_paginator("get_tables").paginate(**kwargs).build_full_result()["TableList"]
 
 
 def run_crawler(crawler: str, *, timeout_minutes: int = 60, retry_seconds: int = 5) -> None:
